use crate::constants::*;
use crate::graphics::vga_color as vga;
use crate::resources::Resources;
use bracket_lib::prelude::*;

pub fn draw(resources: &Resources, bterm: &mut BTerm) {
    // Blue background
    bterm.fill_region(
        Rect {
            x1: SIDEBAR_POS_X,
            x2: SIDEBAR_POS_X + 14,
            y1: SIDEBAR_POS_Y,
            y2: SIDEBAR_POS_Y + 19,
        },
        to_cp437(' '),
        RGB::named(vga::YELLOW_BRIGHT),
        RGB::named(vga::BLUE),
    );

    // Gray number boxes
    (1..17).step_by(3).for_each(|y| {
        bterm.fill_region(
            Rect {
                x1: SIDEBAR_POS_X + 3,
                x2: SIDEBAR_POS_X + 10,
                y1: SIDEBAR_POS_Y + y,
                y2: SIDEBAR_POS_Y + y + 1,
            },
            to_cp437(' '),
            RGB::named(vga::RED),
            RGB::named(vga::WHITE),
        );
    });

    // Stats
    bterm.print(SIDEBAR_POS_X + 4, SIDEBAR_POS_Y, "Score");
    bterm.print(SIDEBAR_POS_X + 4, SIDEBAR_POS_Y + 3, "Level");
    bterm.print_centered_at(
        SIDEBAR_POS_X + 6,
        SIDEBAR_POS_Y + 4,
        resources.level_number + 1,
    );
    bterm.print(SIDEBAR_POS_X + 4, SIDEBAR_POS_Y + 6, "Gems");
    bterm.print(SIDEBAR_POS_X + 4, SIDEBAR_POS_Y + 9, "Whips");
    bterm.print(SIDEBAR_POS_X + 2, SIDEBAR_POS_Y + 12, "Teleports");
    bterm.print(SIDEBAR_POS_X + 4, SIDEBAR_POS_Y + 15, "Keys");

    let stats = &resources.stats;
    bterm.print_centered_at(SIDEBAR_POS_X + 6, SIDEBAR_POS_Y + 1, stats.score * 10);
    bterm.print_centered_at(SIDEBAR_POS_X + 6, SIDEBAR_POS_Y + 7, stats.gems);
    bterm.print_centered_at(
        SIDEBAR_POS_X + 6,
        SIDEBAR_POS_Y + 10,
        if stats.whip_power <= BASE_WHIP_POWER {
            stats.whips.to_string()
        } else {
            format!("{}+{}", stats.whips, stats.whip_power - BASE_WHIP_POWER)
        },
    );
    bterm.print_centered_at(SIDEBAR_POS_X + 6, SIDEBAR_POS_Y + 13, stats.teleports);
    bterm.print_centered_at(SIDEBAR_POS_X + 6, SIDEBAR_POS_Y + 16, stats.keys);

    // Hotkey list
    bterm.print_color(
        SIDEBAR_POS_X + 3,
        SIDEBAR_POS_Y + 18,
        RGB::named(vga::CYAN_BRIGHT),
        RGB::named(vga::RED),
        "OPTIONS",
    );

    bterm.fill_region(
        Rect {
            x1: SIDEBAR_POS_X,
            x2: SIDEBAR_POS_X + 14,
            y1: SIDEBAR_POS_Y + 19,
            y2: SIDEBAR_POS_Y + 25,
        },
        to_cp437(' '),
        RGB::named(vga::WHITE),
        RGB::named(vga::BLUE),
    );

    bterm.fill_region(
        Rect {
            x1: SIDEBAR_POS_X + 3,
            x2: SIDEBAR_POS_X + 4,
            y1: SIDEBAR_POS_Y + 19,
            y2: SIDEBAR_POS_Y + 25,
        },
        to_cp437(' '),
        RGB::named(vga::WHITE_BRIGHT),
        RGB::named(vga::BLUE),
    );

    bterm.print(SIDEBAR_POS_X + 3, SIDEBAR_POS_Y + 19, "Whip");
    bterm.print(SIDEBAR_POS_X + 3, SIDEBAR_POS_Y + 20, "Teleport");
    bterm.print(SIDEBAR_POS_X + 3, SIDEBAR_POS_Y + 21, "Pause");
    if !cfg!(target_family = "wasm") {
        bterm.print(SIDEBAR_POS_X + 3, SIDEBAR_POS_Y + 22, "Quit");
    }
    bterm.print(SIDEBAR_POS_X + 3, SIDEBAR_POS_Y + 23, "Save");
    bterm.print(SIDEBAR_POS_X + 3, SIDEBAR_POS_Y + 24, "Restore");

    if resources.show_debug_info {
        bterm.print_color_right(
            SIDEBAR_POS_X + 14,
            SIDEBAR_POS_Y,
            RGB::named(vga::GREEN_BRIGHT),
            RGB::named(vga::BLACK),
            &format!("{}", bterm.fps),
        );

        let mut debug_string = format!("{}", resources.clock.ticks);

        if resources.speed_time_ticks > 0 {
            debug_string += " SPEED"
        }

        if resources.slow_time_ticks > 0 {
            debug_string += " SLOW"
        }

        bterm.print_color(
            0,
            0,
            RGB::named(vga::YELLOW_BRIGHT),
            RGB::named(vga::BLACK),
            &debug_string,
        );
    }
}
