use crate::constants::*;

pub enum Difficulty {
    Novice,
    Experienced,
    Advanced,
    Secret,
}

#[derive(Debug, Eq, PartialEq, Copy, Clone, Hash)]
pub struct DifficultySettings {
    pub value: u32,
    pub starting_gems: u32,
    pub starting_whips: u32,
    pub starting_teleports: u32,
    pub starting_keys: u32,
    pub starting_whip_power: u32,
}

impl From<&Difficulty> for DifficultySettings {
    fn from(difficulty: &Difficulty) -> Self {
        match difficulty {
            Difficulty::Novice => NOVICE,
            Difficulty::Experienced => EXPERIENCED,
            Difficulty::Advanced => ADVANCED,
            Difficulty::Secret => SECRET,
        }
    }
}

const SECRET: DifficultySettings = DifficultySettings {
    value: 9,
    starting_gems: 250,
    starting_whips: 100,
    starting_teleports: 50,
    starting_keys: 1,
    starting_whip_power: BASE_WHIP_POWER + 1,
};

const NOVICE: DifficultySettings = DifficultySettings {
    value: 8,
    starting_gems: 20,
    starting_whips: 10,
    starting_teleports: 0,
    starting_keys: 0,
    starting_whip_power: BASE_WHIP_POWER,
};

const EXPERIENCED: DifficultySettings = DifficultySettings {
    value: 5,
    starting_gems: 15,
    starting_whips: 0,
    starting_teleports: 0,
    starting_keys: 0,
    starting_whip_power: BASE_WHIP_POWER,
};

const ADVANCED: DifficultySettings = DifficultySettings {
    value: 2,
    starting_gems: 10,
    starting_whips: 0,
    starting_teleports: 0,
    starting_keys: 0,
    starting_whip_power: BASE_WHIP_POWER,
};
